/*
	APL Restful ARDBC Plugin
    Copyright 2015-2018  LJ LongWing - A Programming Legacy (http://remedylegacy.com)

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

		http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
 */

package com.apl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.json.JSONArray;
import org.json.JSONObject;

import com.bmc.arsys.pluginsvr.plugins.ARVendorField;

public class JSONParser {
	private Map<String, List<String>> arrayElementMap = new TreeMap<String, List<String>>();
	private int currentLevel = 0;
	private String elementPrefix = "ROOT:";
	private List<String> fieldNameList = new ArrayList<String>();
	private Map<String, String> globalElementMap = new TreeMap<String, String>();
	private boolean inRecord = false;
	private List<Map<String, String>> recordElementList = new ArrayList<Map<String, String>>();
	private Map<String, String> recordElementMap = new TreeMap<String, String>();
	private int recordIndicatorLevel = -1;
	private List<Integer> skipLevelList = new ArrayList<Integer>();
	private String tableRecordIndicator = "ROOT";

	public JSONParser(List<ARVendorField> fieldList, String tableRecordIndicator, int recordIndicatorLevel,
			List<Integer> skipLevelList) {
		super();
		if (fieldList != null) {
			for (ARVendorField field : fieldList) {
				fieldNameList.add(field.getName());
			}
		}

		if (tableRecordIndicator != null) {
			this.tableRecordIndicator = tableRecordIndicator;
		}

		this.recordIndicatorLevel = recordIndicatorLevel;

		if (skipLevelList != null) {
			this.skipLevelList = skipLevelList;
		}
	}

	private void addKey(String element, String value) {
		if (inRecord) {
			processMapElements(recordElementMap, element, value);
		} else {
			processMapElements(globalElementMap, element, value);
		}
	}

	public Map<String, String> getGlobalElementMap() {
		return globalElementMap;
	}

	public List<Map<String, String>> getRecordElementList() {
		return recordElementList;
	}

	public Map<String, String> getRecordElementMap() {
		return recordElementMap;
	}

	public void parse(String jsonText, String arrayDelimiter) {
		processValue(jsonText);
		List<Map<String, String>> map = Functions.processArrays(arrayElementMap, globalElementMap, recordElementMap,
				fieldNameList, arrayDelimiter);
		if (map != null && map.get(0) != null) {
			globalElementMap = map.get(0);
		}
		if (map != null && map.get(1) != null) {
			recordElementMap = map.get(1);
		}
	}

	private void processMapElements(Map<String, String> map, String element, String value) {
		value = value.trim();
		if (!map.containsKey(element)) {
			map.put(element, value);
		} else {
			List<String> currentValueList = arrayElementMap.get(element);
			if (currentValueList == null) {
				String curValue = map.get(element);
				currentValueList = new ArrayList<String>();
				currentValueList.add(curValue);
			}
			currentValueList.add(value);
			arrayElementMap.put(element, currentValueList);
		}
	}

	private void processValue(String string) {
		string = string.trim();
		if (string.startsWith("{")) {
			startRecord();

			JSONObject jsonObject = new JSONObject(string);

			Set<String> keySet = jsonObject.keySet();
			for (String key : keySet) {
				if (!jsonObject.isNull(key)) {
					Object o = jsonObject.get(key);
					if (o instanceof JSONObject || o instanceof JSONArray) {
						currentLevel++;
						if (!skipLevelList.contains(currentLevel)) {
							elementPrefix = Functions.addElementPrefix(elementPrefix, key);
						}
						processValue(o.toString());
						if (!skipLevelList.contains(currentLevel)) {
							elementPrefix = Functions.removeElementSuffix(elementPrefix, key);
						}
						currentLevel--;
					} else {
						addKey(elementPrefix + key, String.valueOf(o));
					}
				}
			}

			stopRecord();

		} else if (string.startsWith("[")) {
			JSONArray jsonArray = new JSONArray(string);
			for (Object o : jsonArray) {
				if (o instanceof JSONObject) {
					startRecord();
					processValue(o.toString());
					stopRecord();
				} else {
					String element = elementPrefix.substring(0, elementPrefix.length() - 1);
					if (element.equals(tableRecordIndicator) || currentLevel == recordIndicatorLevel) {
						startRecord();
						addKey(element, o.toString());
						stopRecord();
					} else {
						addKey(element, o.toString());
					}
				}
			}
		} else {
			System.out.println("JSON Document doesn't start with expected { or [, please correct: " + string);
		}
	}

	private void startRecord() {
		String curElementStack = elementPrefix.substring(0, elementPrefix.length() - 1);
		if (!inRecord && (curElementStack.equals(tableRecordIndicator) || currentLevel == recordIndicatorLevel)) {
			inRecord = true;
			recordElementMap = new TreeMap<String, String>();
		}
	}

	private void stopRecord() {
		String curElementStack = elementPrefix.substring(0, elementPrefix.length() - 1);
		if (inRecord && (curElementStack.equals(tableRecordIndicator) || currentLevel == recordIndicatorLevel)) {
			inRecord = false;
			recordElementList.add(recordElementMap);
		}
	}
}
